package flashbuy

import (
	"encoding/json"
	"errors"
	"net/http"
	"spaceofcheng/services/controllers"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/flashbuy"
	"spaceofcheng/services/utils"
)

// CaseController 信息
type FlashBuyController struct {
	dao *flashbuy.FlashBuyServ
	controllers.BaseController
}

// Constructor 初始化 Controller
// @Title Constructor
// @Description 初始化 Controller, 系统自动调用
func (c *FlashBuyController) Constructor() {
	c.dao = flashbuy.NewFlashBuyServ(c.Context)
}
func (c *FlashBuyController) GetFlashBuyList() {
	caseIDs := c.GetString("caseid")
	if caseIDs == "" {
		cases := c.Context.Get("cases").([]model.SysUserCase)
		caseIDs = c.GetCaseIDs(cases)
	}
	flashBuyName := c.GetString("flashbuyname")
	flashBuyStatus := c.GetString("flashbuystatus")
	page, _ := c.GetInt("page")
	pageSize, _ := c.GetInt("pagesize")
	list, err := c.dao.GetFlashBuyList(caseIDs, flashBuyName, flashBuyStatus, page, pageSize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(list)
}

func (c *FlashBuyController) GetCustomerFlashBuyByCustomerId() {
	userRaw := c.Context.Get("customer")
	if userRaw == nil {
		c.ResponseError(errors.New("系统内部错误"), http.StatusInternalServerError)
	}

	user := userRaw.(model.TaCustomer)
	customerId := user.CustomerId
	page, _ := c.GetInt("page")
	pageSize, _ := c.GetInt("pagesize")
	list, err := c.dao.GetCustomerFlashBuyByCustomerId(customerId, page, pageSize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(list)
}

func (c *FlashBuyController) GetCustomerFlashBuyList() {
	flashBuyId := c.GetString(":flashBuyId")
	phone := c.GetString("phone")
	page, _ := c.GetInt("page")
	pageSize, _ := c.GetInt("pagesize")
	list, err := c.dao.GetCustomerFlashBuyById(flashBuyId, phone, page, pageSize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(list)

}

func (c *FlashBuyController) GetFlashBuyById() {
	flashBuyId := c.GetString(":flashBuyId")
	flashBuy, err := c.dao.GetFlashBuyById(flashBuyId)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(flashBuy)
}

func (c *FlashBuyController) GetCustomerFlashBuyId() {
	flashBuyId := c.GetString(":customerFlashBuyId")
	flashBuy, err := c.dao.GetCustomerFlashBuyId(flashBuyId)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(flashBuy)
}

func (c *FlashBuyController) SaveFlashBuy() {
	jsnStr := c.GetString("info")
	if jsnStr == "" {
		c.ResponseError(errors.New("未接收到保存内容"))
	}
	info := model.TaFlashBuy{}
	if err := json.Unmarshal([]byte(jsnStr), &info); err != nil {
		utils.LogError("抢购数据转换失败: " + err.Error())
		c.ResponseError(err)
	}
	newFlashBuy, err := c.dao.SaveFlashBuy(info)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(newFlashBuy)
}

func (c *FlashBuyController) DeleteFlashBuy() {
	flashBuyId := c.GetString(":flashBuyId")
	err := c.dao.DeleteFlashBuy(flashBuyId)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("删除成功")
}

func (c *FlashBuyController) UpdateFlashBuy() {
	flashBuyId := c.GetString(":flashBuyId")
	flashBuyStatus := c.GetString(":flashBuyStatus")
	err := c.dao.UpdateFlashBuy(flashBuyId, flashBuyStatus)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("修改成功")
}

func (c *FlashBuyController) VerifyCustomerFlashBuyList() {
	qrcode := c.GetString(":qrcode")
	caseids := c.GetString(":caseId")
	customerFlashBuy, err := c.dao.VerifyCustomerFlashBuyList(qrcode, caseids)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(customerFlashBuy)
}
func (c *FlashBuyController) Verify() {
	customerFlashBuyId := c.GetString(":customerFlashBuyId")
	err := c.dao.Verify(customerFlashBuyId)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("核销成功")
}

// FlashBuy 抢购
func (c *FlashBuyController) FlashBuy() {
	id := c.GetString(":id")

	c.Context.Set("clienturl", c.Configer[controllers.WeChatConf].String("flashBuyNoticeURL"))
	c.Context.Set("tplid", c.Configer[controllers.WeChatConf].String("luckdrawMessageTplID"))

	err := c.dao.FlashBuy(id)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("恭喜您！抢购成功！")
}

func (c *FlashBuyController) GetFlashModelList() {
	model, err := c.dao.GetFlashModelList()
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(model)
}
